@@ -43,8 +43,8 @@
 struct pkg_add_db {
 	struct pkgdb *db;
 	pkgs_t localpkgs;
-	struct pkghash *system_shlibs;
-	bool local_scaned;
+	charv_t system_shlibs;
+	bool local_scanned;
 	bool ignore_compat32;
 	bool pkgbase;
 };
@@ -247,7 +247,7 @@ attempt_to_merge(int rootfd, struct pkg_config_file *rcf, struct pkg *local,
 	newconf = xstring_new();
 	if (merge_3way(lcf->content, localconf, rcf->content, newconf) != 0) {
 		xstring_free(newconf);
-		pkg_emit_error("Impossible to merge configuration file");
+		pkg_emit_error("Impossible to merge configuration file: %s", rcf->path);
 	} else {
 		char *conf = xstring_get(newconf);
 		rcf->newcontent = conf;
@@ -1081,61 +1081,132 @@ pkg_extract_finalize(struct pkg *pkg, tempdirs_t *tempdirs)
 	return (EPKG_OK);
 }
 
-static bool
-should_append_pkg(pkgs_t *localpkgs, struct pkg *p)
+bool
+append_pkg_if_newer(pkgs_t *localpkgs, struct pkg *p)
 {
-	/* only keep the highest version is we fine one */
-	vec_foreach(*localpkgs, i) {
-		struct pkg *lp = localpkgs->d[i];
-		if (strcmp(lp->name, p->name) == 0) {
-			if (pkg_version_cmp(lp->version, p->version) == -1) {
-				pkg_free(localpkgs->d[i]);
-				localpkgs->d[i] = p;
-				return (true);
-			}
-			return (false);
+	/* only keep the highest version if we find one */
+	struct pkg **lp = pkgs_insert_sorted(localpkgs, p);
+	if (lp != NULL) {
+		if (pkg_version_cmp((*lp)->version, p->version) == -1) {
+			pkg_free(*lp);
+			*lp = p;
+			return (true);
 		}
+		return (false);
 	}
-	/* none found we should append */
-	vec_push(localpkgs, p);
 	return (true);
 }
 
 struct localhashes {
-	struct pkghash *shlibs_provides;
-	struct pkghash *provides;
+	kvlist_t shlibs_provides;
+	kvlist_t provides;
 };
 
+static FILE *
+open_cache_read(void)
+{
+	FILE *fp;
+	int cfd, fd;
+
+	cfd = pkg_get_cachedirfd();
+	if (cfd == -1)
+		return (NULL);
+	if ((fd = openat(cfd, "pkg_add_cache", O_RDONLY)) == -1)
+		return (NULL);
+	if ((fp = fdopen(fd, "r")) == NULL)
+		close(fd);
+	return (fp);
+}
+
+static FILE *
+open_cache_write(void)
+{
+	FILE *fp;
+	int cfd, fd;
+
+	cfd = pkg_get_cachedirfd();
+	if (cfd == -1) {
+		if (errno == ENOENT) {
+			if (pkg_mkdirs(ctx.cachedir) != EPKG_OK)
+				return (NULL);
+		} else {
+			return (NULL);
+		}
+		cfd = pkg_get_cachedirfd();
+		if (cfd == -1)
+			return (NULL);
+	}
+	if ((fd = openat(cfd, "pkg_add_cache", O_WRONLY|O_TRUNC)) == -1)
+		return (NULL);
+	if ((fp = fdopen(fd, "w")) == NULL)
+		close(fd);
+	return (fp);
+}
+
 static void
 scan_local_pkgs(struct pkg_add_db *db, bool fromstdin, struct localhashes *l, const char *bd, const char *ext)
 {
 	if (!fromstdin) {
-		if (!db->local_scaned) {
-			glob_t g;
-			char *pattern;
-			xasprintf(&pattern, "%s/*%s" , bd, ext);
-			db->local_scaned = true;
-			if (glob(pattern, 0, NULL, &g) == 0) {
-				for (int i = 0; i <g.gl_pathc; i++) {
-					struct pkg *p = NULL;
-					if (pkg_open(&p, g.gl_pathv[i],
-					    PKG_OPEN_MANIFEST_COMPACT) == EPKG_OK) {
-						if (should_append_pkg(&db->localpkgs, p)) {
-							p->repopath = xstrdup(g.gl_pathv[i]);
+		if (!db->local_scanned) {
+			bool package_building = (getenv("PACKAGE_BUILDING") != NULL);
+			bool cache_exist = false;
+			if (package_building) {
+				FILE * fp;
+				if ((fp = open_cache_read()) != NULL) {
+					cache_exist = true;
+					char *line = NULL;
+					size_t linecap = 0;
+					ssize_t linelen;
+					while ((linelen = getline(&line, &linecap, fp)) > 0) {
+						struct pkg * p = NULL;
+						pkg_new(&p, PKG_FILE);
+						pkg_parse_manifest(p, line, linelen);
+						vec_push(&db->localpkgs, p);
+					}
+					free(line);
+				}
+			}
+			if (db->localpkgs.len == 0) {
+				glob_t g;
+				char *pattern;
+				xasprintf(&pattern, "%s/*%s" , bd, ext);
+				db->local_scanned = true;
+				if (glob(pattern, 0, NULL, &g) == 0) {
+					for (int i = 0; i <g.gl_pathc; i++) {
+						struct pkg *p = NULL;
+						if (pkg_open(&p, g.gl_pathv[i],
+									PKG_OPEN_MANIFEST_COMPACT) == EPKG_OK) {
+							if (append_pkg_if_newer(&db->localpkgs, p)) {
+								p->repopath = xstrdup(g.gl_pathv[i]);
+							}
 						}
 					}
 				}
+				globfree(&g);
+				free(pattern);
+			}
+			if (package_building && !cache_exist) {
+				FILE *fp = open_cache_write();
+				if (fp != NULL) {
+					vec_foreach(db->localpkgs, i) {
+						pkg_emit_manifest_file(db->localpkgs.d[i], fp, PKG_MANIFEST_EMIT_COMPACT);
+						fputs("\n", fp);
+					}
+					fclose(fp);
+				}
 			}
-			globfree(&g);
-			free(pattern);
 		}
 		vec_foreach(db->localpkgs, i) {
 			struct pkg *p = db->localpkgs.d[i];
 			vec_foreach(p->shlibs_provided, j) {
-				pkghash_safe_add(l->shlibs_provides, p->shlibs_provided.d[j], xstrdup(p->repopath), free);
+				struct pkg_kv *kv = pkg_kv_new(p->shlibs_provided.d[j], p->repopath);
+				if (pkg_kv_insert_sorted(&l->shlibs_provides, kv) != NULL)
+					pkg_kv_free(kv);
 			}
 			vec_foreach(p->provides, j) {
-				pkghash_safe_add(l->provides, p->provides.d[j], xstrdup(p->repopath), free);
+				struct pkg_kv *kv = pkg_kv_new(p->provides.d[j], p->repopath);
+				if (pkg_kv_insert_sorted(&l->provides, kv) != NULL)
+					pkg_kv_free(kv);
 			}
 		}
 	}
@@ -1144,10 +1215,9 @@ scan_local_pkgs(struct pkg_add_db *db, bool fromstdin, struct localhashes *l, co
 static const char *
 _localpkgs_get(pkgs_t *pkgs, const char *name)
 {
-	vec_foreach(*pkgs, i) {
-		if (STREQ(pkgs->d[i]->name, name))
-			return (pkgs->d[i]->repopath);
-	}
+	struct pkg **lp = pkgs_search(pkgs, name);
+	if (lp != NULL)
+		return ((*lp)->repopath);
 	return (NULL);
 }
 
@@ -1265,17 +1335,17 @@ pkg_add_check_pkg_archive(struct pkg_add_db *db, struct pkg *pkg,
 	}
 
 	vec_foreach(pkg->shlibs_required, i) {
-		const char *s = pkg->shlibs_required.d[i];
+		char *s = pkg->shlibs_required.d[i];
 		pkg_debug(2, "%s requires %s", pkg->name, s);
-		if (!db->pkgbase && db->system_shlibs == NULL) {
+		if (!db->pkgbase && db->system_shlibs.len == 0) {
 			int ret;
 			ret = scan_system_shlibs(&db->system_shlibs, ctx.pkg_rootdir);
 			if (ret == EPKG_NOCOMPAT32)
 				db->ignore_compat32 = true;
 		}
-		if (pkghash_get(db->system_shlibs, s) != NULL)
+		if (charv_search(&db->system_shlibs, s) != NULL)
 			continue;
-		pkghash_entry *founddep = NULL;
+		const struct pkg_kv *founddep = NULL;
 		if (pkgdb_is_shlib_provided(db->db, s))
 			continue;
 
@@ -1289,7 +1359,7 @@ pkg_add_check_pkg_archive(struct pkg_add_db *db, struct pkg *pkg,
 			scan_local_pkgs(db, fromstdin, &l, bd, ext);
 			scanned = true;
 		}
-		if ((founddep = pkghash_get(l.shlibs_provides, s)) == NULL) {
+		if ((founddep = pkg_kv_search(&l.shlibs_provides, s)) == NULL) {
 			pkg_emit_error("Missing shlib dependency: %s", s);
 			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
 				goto cleanup;
@@ -1310,8 +1380,8 @@ pkg_add_check_pkg_archive(struct pkg_add_db *db, struct pkg *pkg,
 	}
 
 	vec_foreach(pkg->requires, i) {
-		const char *s = pkg->requires.d[i];
-		pkghash_entry *founddep = NULL;
+		char *s = pkg->requires.d[i];
+		const struct pkg_kv *founddep = NULL;
 		if (pkgdb_is_provided(db->db, s))
 			continue;
 
@@ -1325,7 +1395,7 @@ pkg_add_check_pkg_archive(struct pkg_add_db *db, struct pkg *pkg,
 			scan_local_pkgs(db, fromstdin, &l, bd, ext);
 			scanned = true;
 		}
-		if ((founddep = pkghash_get(l.provides, s)) == NULL) {
+		if ((founddep = pkg_kv_search(&l.provides, s)) == NULL) {
 			pkg_emit_error("Missing require dependency: %s", s);
 			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
 				goto cleanup;
@@ -1333,7 +1403,7 @@ pkg_add_check_pkg_archive(struct pkg_add_db *db, struct pkg *pkg,
 		}
 		if ((flags & PKG_ADD_UPGRADE) == 0 &&
 				access(founddep->value, F_OK) == 0) {
-			pkg_debug(2, "Installing %s because of requires: %s", founddep, s);
+			pkg_debug(2, "Installing %s because of requires: %s", founddep->value, s);
 			ret = pkg_add_common(db, founddep->value, PKG_ADD_AUTOMATIC, location, NULL, NULL, NULL);
 
 			if (ret != EPKG_OK)
@@ -1347,8 +1417,8 @@ pkg_add_check_pkg_archive(struct pkg_add_db *db, struct pkg *pkg,
 
 	retcode = EPKG_OK;
 cleanup:
-	pkghash_destroy(l.provides);
-	pkghash_destroy(l.shlibs_provides);
+	vec_free_and_free(&l.shlibs_provides, pkg_kv_free);
+	vec_free_and_free(&l.provides, pkg_kv_free);
 	pkg_emit_add_deps_finished(pkg);
 
 	return (retcode);
@@ -1393,7 +1463,7 @@ pkg_add_cleanup_old(struct pkgdb *db, struct pkg *old, struct pkg *new, struct t
 					const char *libname;
 					libname = strrchr(f->path, '/');
 					if (libname != NULL &&
-					    charv_contains(&old->shlibs_provided, libname+1, false)) {
+					    charv_search(&old->shlibs_provided, libname+1) != NULL) {
 						backup_library(db, old, f->path);
 					}
 				}
@@ -1453,15 +1523,13 @@ pkg_add_common(struct pkg_add_db *db, const char *path, unsigned flags,
 	int			 retcode = EPKG_OK;
 	int			 ret;
 	int			 nfiles;
-	tempdirs_t		 tempdirs;
+	tempdirs_t		 tempdirs = vec_init();
 	struct pkg_add_context	 context;
 
 	memset(&context, 0, sizeof(context));
 
 	assert(path != NULL);
 
-	vec_init(&tempdirs);
-
 	/*
 	 * Open the package archive file, read all the meta files and set the
 	 * current archive_entry to the first non-meta file.
@@ -1689,13 +1757,13 @@ pkg_add(struct pkgdb *db, const char *path, unsigned flags,
 
 	memset(&padb, 0, sizeof(padb));
 	padb.db = db;
-	padb.local_scaned = false;
+	padb.local_scanned = false;
 	padb.ignore_compat32 = false;
 	padb.pkgbase = pkgdb_file_exists(db, "/usr/bin/uname");
 
 	ret = pkg_add_common(&padb, path, flags, location, NULL, NULL, NULL);
 	vec_free_and_free(&padb.localpkgs, pkg_free);
-	pkghash_destroy(padb.system_shlibs);
+	vec_free_and_free(&padb.system_shlibs, free);
 	return (ret);
 }
 
@@ -1785,18 +1853,16 @@ pkg_add_fromdir(struct pkg *pkg, const char *src, struct pkgdb *db __unused)
 	struct group *gr, grent;
 	int err, fd, fromfd;
 	int retcode;
-	hardlinks_t hardlinks;
+	hardlinks_t hardlinks = vec_init();
 	const char *path;
 	char buffer[1024];
 	size_t link_len;
 	bool install_as_user;
-	tempdirs_t tempdirs;
+	tempdirs_t tempdirs = vec_init();
 	struct pkg_add_context context;
 
 	memset(&context, 0, sizeof(context));
 
-	vec_init(&hardlinks);
-	vec_init(&tempdirs);
 	install_as_user = (getenv("INSTALL_AS_USER") != NULL);
 
 	fromfd = open(src, O_DIRECTORY);
