@@ -67,7 +67,6 @@
 #include "private/pkg.h"
 #include "private/pkgdb.h"
 #include "private/pkg_jobs.h"
-#include "tllist.h"
 
 extern struct pkg_ctx ctx;
 
@@ -81,7 +80,7 @@ struct pkg_jobs_locked {
 	void *context;
 };
 static __thread struct pkg_jobs_locked *pkgs_job_lockedpkg;
-typedef tll(int64_t) candidates_t;
+typedef vec_t(int64_t) candidates_t;
 
 #define IS_DELETE(j) ((j)->type == PKG_JOBS_DEINSTALL || (j)->type == PKG_JOBS_AUTOREMOVE)
 
@@ -119,8 +118,7 @@ pkg_jobs_set_flags(struct pkg_jobs *j, pkg_flags flags)
 int
 pkg_jobs_set_repository(struct pkg_jobs *j, const char *ident)
 {
-	c_charv_t idents;
-	vec_init(&idents);
+	c_charv_t idents = vec_init();
 	if (ident != NULL)
 		vec_push(&idents, ident);
 	return (pkg_jobs_set_repositories(j, &idents));
@@ -206,10 +204,10 @@ pkg_jobs_free(struct pkg_jobs *j)
 	j->request_delete = NULL;
 
 	pkg_jobs_universe_free(j->universe);
-	tll_free_and_free(j->jobs, free);
+	vec_free_and_free(&j->jobs, free);
 	LL_FREE(j->patterns, pkg_jobs_pattern_free);
 	if (j->triggers.cleanup != NULL) {
-		tll_free_and_free(*j->triggers.cleanup, trigger_free);
+		vec_free_and_free(j->triggers.cleanup, trigger_free);
 		free(j->triggers.cleanup);
 	}
 	if (j->triggers.dfd != -1)
@@ -218,7 +216,7 @@ pkg_jobs_free(struct pkg_jobs *j)
 		ucl_object_unref(j->triggers.schema);
 	pkghash_destroy(j->orphaned);
 	pkghash_destroy(j->notorphaned);
-	pkghash_destroy(j->system_shlibs);
+	vec_free_and_free(&j->system_shlibs, free);
 	free(j);
 }
 
@@ -308,26 +306,28 @@ pkg_jobs_iter(struct pkg_jobs *j, void **iter,
 {
 	struct pkg_solved *s;
 	struct {
-		typeof(*(j->jobs.head)) *it;
+		pkg_solved_list *list;
+		size_t pos;
 	} *t;
 	t = *iter;
 	if (*iter == NULL) {
 		t = xcalloc(1, sizeof(*t));
 		*iter = t;
-	} else if (t->it == NULL) {
+	} else if (t->pos >= t->list->len) {
 			free(t);
 			return (false);
 	}
 
-	if (tll_length(j->jobs) == 0)
+	if (j->jobs.len == 0)
 		return (false);
-	if (t->it == NULL)
-		t->it = j->jobs.head;
-	s = t->it->item;
+	if (t->list == NULL) {
+		t->list = &j->jobs;
+		t->pos = 0;
+	}
+	s = t->list->d[t->pos++];
 	*new = s->items[0]->pkg;
 	*old = s->items[1] ? s->items[1]->pkg : NULL;
 	*type = s->type;
-	t->it = t->it->next;
 	return (true);
 }
 
@@ -484,7 +484,7 @@ delete_process_provides(struct pkg_jobs *j, struct pkg *lp, const char *provide,
 	bool ret = true;
 
 	/* check for pkgbase shlibs and provides */
-	if (pkghash_get(j->system_shlibs, provide) != NULL)
+	if (charv_search(&j->system_shlibs, provide) != NULL)
 		return (ret);
 	/* if something else to provide the same thing we can safely delete */
 	lit = provideq(j->db, provide);
@@ -537,11 +537,9 @@ pkg_jobs_process_delete_request(struct pkg_jobs *j)
 	struct pkg_dep *d = NULL;
 	struct pkg *lp;
 	int rc = EPKG_OK;
-	pkgs_t to_process;
+	pkgs_t to_process = vec_init();
 	pkghash_it it;
 
-	vec_init(&to_process);
-
 	if (force)
 		return (EPKG_OK);
 
@@ -1052,7 +1050,7 @@ pkg_jobs_find_remote_pattern(struct pkg_jobs *j, struct job_pattern *jp)
 }
 
 bool
-pkg_jobs_need_upgrade(struct pkghash *system_shlibs, struct pkg *rp, struct pkg *lp)
+pkg_jobs_need_upgrade(charv_t *system_shlibs, struct pkg *rp, struct pkg *lp)
 {
 	int ret, ret1, ret2;
 	struct pkg_option *lo = NULL, *ro = NULL;
@@ -1235,11 +1233,11 @@ pkg_jobs_need_upgrade(struct pkghash *system_shlibs, struct pkg *rp, struct pkg
 		if (STREQ(lp->shlibs_required.d[i], rp->shlibs_required.d[j]))
 				continue;
 		if (system_shlibs != NULL) {
-			if (pkghash_get(system_shlibs, lp->shlibs_required.d[i]) != NULL) {
+			if (charv_search(system_shlibs, lp->shlibs_required.d[i]) != NULL) {
 				j--;
 				continue;
 			}
-			if (pkghash_get(system_shlibs, rp->shlibs_required.d[j]) != NULL) {
+			if (charv_search(system_shlibs, rp->shlibs_required.d[j]) != NULL) {
 				i++;
 				continue;
 			}
@@ -1366,8 +1364,8 @@ pkg_jobs_set_deinstall_reasons(struct pkg_jobs *j)
 	struct pkg_job_request *jreq;
 	struct pkg *req_pkg, *pkg;
 
-	tll_foreach(j->jobs, it) {
-		sit = it->item;
+	vec_foreach(j->jobs, i) {
+		sit = j->jobs.d[i];
 		jreq = pkg_jobs_find_deinstall_request(sit->items[0], j, 0);
 		if (jreq != NULL && jreq->item->unit != sit->items[0]) {
 			req_pkg = jreq->item->pkg;
@@ -1507,7 +1505,7 @@ pkg_jobs_find_install_candidates(struct pkg_jobs *j)
 
 		if ((j->flags & PKG_FLAG_FORCE) ||
 						pkg_jobs_check_remote_candidate(j, pkg)) {
-			tll_push_front(*candidates, pkg->id);
+			vec_push(candidates, pkg->id);
 		}
 		pkg_free(pkg);
 		pkg = NULL;
@@ -1534,15 +1532,15 @@ jobs_solve_full_upgrade(struct pkg_jobs *j)
 	assert(!j->solved);
 
 	candidates = pkg_jobs_find_install_candidates(j);
-	jcount = tll_length(*candidates);
+	jcount = candidates->len;
 
 	pkg_emit_progress_start("Checking for upgrades (%zd candidates)",
 			jcount);
 
-	tll_foreach(*candidates, c) {
+	vec_foreach(*candidates, i) {
 		pkg_emit_progress_tick(++elt_num, jcount);
 		sqlite3_snprintf(sizeof(sqlbuf), sqlbuf, " WHERE p.id=%" PRId64,
-		    c->item);
+		    candidates->d[i]);
 		if ((it = pkgdb_query_cond(j->db, sqlbuf, NULL, MATCH_ALL)) == NULL)
 			return (EPKG_FATAL);
 
@@ -1554,7 +1552,7 @@ jobs_solve_full_upgrade(struct pkg_jobs *j)
 		pkg_free(pkg);
 		pkgdb_it_free(it);
 	}
-	tll_free(*candidates);
+	vec_free(candidates);
 	free(candidates);
 	pkg_emit_progress_tick(jcount, jcount);
 
@@ -1880,7 +1878,7 @@ pkg_jobs_solve(struct pkg_jobs *j)
 {
 	int ret;
 
-	if (j->system_shlibs == NULL) {
+	if (j->system_shlibs.len == 0) {
 		/* If /usr/bin/uname is in the pkg database, we are targeting
 		 * a pkgbase system and should rely on the pkgbase packages to
 		 * provide system shlibs. */
@@ -1903,10 +1901,10 @@ pkg_jobs_solve(struct pkg_jobs *j)
 	 * conflicts if we can check for and solve conflicts without first
 	 * needing to fetch.
 	 */
-	tll_foreach(j->jobs, job) {
+	vec_foreach(j->jobs, i) {
 		struct pkg *p;
 
-		p = ((struct pkg_solved *)job->item)->items[0]->pkg;
+		p = ((struct pkg_solved *)j->jobs.d[i])->items[0]->pkg;
 		if (p->type != PKG_REMOTE)
 			continue;
 
@@ -1925,7 +1923,7 @@ pkg_jobs_solve(struct pkg_jobs *j)
 			rc = pkg_jobs_check_conflicts(j);
 			if (rc == EPKG_CONFLICT) {
 				/* Cleanup results */
-				tll_free_and_free(j->jobs, free);
+				vec_free_and_free(&j->jobs, free);
 				has_conflicts = true;
 				pkg_jobs_solve(j);
 			}
@@ -1943,7 +1941,7 @@ pkg_jobs_count(struct pkg_jobs *j)
 {
 	assert(j != NULL);
 
-	return (tll_length(j->jobs));
+	return (j->jobs.len);
 }
 
 int
@@ -2089,9 +2087,9 @@ pkg_jobs_execute(struct pkg_jobs *j)
 	if (retcode != EPKG_OK)
 		return (retcode);
 
-	total_actions = tll_length(j->jobs);
-	tll_foreach(j->jobs, _p) {
-		struct pkg_solved *ps = _p->item;
+	total_actions = j->jobs.len;
+	vec_foreach(j->jobs, i) {
+		struct pkg_solved *ps = j->jobs.d[i];
 
 		pkg_emit_new_action(++current_action, total_actions);
 		switch (ps->type) {
@@ -2183,7 +2181,7 @@ pkg_jobs_apply(struct pkg_jobs *j)
 						rc = pkg_jobs_check_conflicts(j);
 						if (rc == EPKG_CONFLICT) {
 							/* Cleanup results */
-							tll_free_and_free(j->jobs, free);
+							vec_free_and_free(&j->jobs, free);
 							has_conflicts = true;
 							rc = pkg_jobs_solve(j);
 						}
@@ -2244,8 +2242,8 @@ pkg_jobs_fetch(struct pkg_jobs *j)
 		cachedir = j->destdir;
 
 	/* check for available size to fetch */
-	tll_foreach(j->jobs, _p) {
-		struct pkg_solved *ps = _p->item;
+	vec_foreach(j->jobs, i) {
+		struct pkg_solved *ps = j->jobs.d[i];
 		if (ps->type != PKG_SOLVED_DELETE && ps->type != PKG_SOLVED_UPGRADE_REMOVE) {
 			p = ps->items[0]->pkg;
 			if (p->type != PKG_REMOTE)
@@ -2310,8 +2308,8 @@ pkg_jobs_fetch(struct pkg_jobs *j)
 		return (EPKG_OK); /* don't download anything */
 
 	/* Fetch */
-	tll_foreach(j->jobs, _p) {
-		struct pkg_solved *ps = _p->item;
+	vec_foreach(j->jobs, i) {
+		struct pkg_solved *ps = j->jobs.d[i];
 		if (ps->type != PKG_SOLVED_DELETE && ps->type != PKG_SOLVED_UPGRADE_REMOVE) {
 			p = ps->items[0]->pkg;
 			if (p->type != PKG_REMOTE)
@@ -2342,8 +2340,8 @@ pkg_jobs_check_conflicts(struct pkg_jobs *j)
 	pkg_emit_integritycheck_begin();
 	j->conflicts_registered = 0;
 
-	tll_foreach(j->jobs, _p) {
-		struct pkg_solved *ps = _p->item;
+	vec_foreach(j->jobs, i) {
+		struct pkg_solved *ps = j->jobs.d[i];
 		if (ps->type == PKG_SOLVED_DELETE || ps->type == PKG_SOLVED_UPGRADE_REMOVE) {
 			continue;
 		}
