@@ -28,8 +28,6 @@
 #include "private/fetch.h"
 #include "pkghash.h"
 
-#define UCL_COUNT(obj) ((obj)?((obj)->len):0)
-
 #define PKG_NUM_SCRIPTS 9
 #define PKG_NUM_LUA_SCRIPTS 5
 
@@ -241,9 +239,13 @@ struct pkg {
 	pkg_t		 type;
 	struct pkg_repo		*repo;
 };
-typedef tll(struct pkg *) pkg_chain_t;
 typedef vec_t(struct pkg *) pkgs_t;
 
+DEFINE_VEC_INSERT_SORTED_PROTO(pkgs_t, pkgs, struct pkg *);
+struct pkg **pkgs_search(pkgs_t *, char *);
+void pkgs_sort(pkgs_t *);
+bool append_pkg_if_newer(pkgs_t *pkgs, struct pkg *p);
+
 typedef enum {
 	SCRIPT_UNKNOWN = 0,
 	SCRIPT_SHELL,
@@ -267,7 +269,7 @@ struct trigger {
 	} cleanup;
 	pkghash *matched;
 };
-typedef tll(struct trigger *) trigger_t;
+typedef vec_t(struct trigger *) trigger_t;
 
 struct triggers {
 	ucl_object_t *schema;
@@ -677,6 +679,9 @@ void pkg_list_free(struct pkg *, pkg_list);
 
 struct pkg_kv *pkg_kv_new(const char *key, const char *val);
 void pkg_kv_free(struct pkg_kv *);
+struct pkg_kv *pkg_kv_search(kvlist_t *, char *);
+void pkg_kv_sort(kvlist_t *);
+DEFINE_VEC_INSERT_SORTED_PROTO(kvlist_t, pkg_kv, struct pkg_kv *);
 
 void pkg_dep_free(struct pkg_dep *);
 void pkg_file_free(struct pkg_file *);
@@ -864,7 +869,7 @@ int pkg_parse_manifest_ucl(struct pkg *pkg, ucl_object_t *o);
 int pkg_get_reposdirfd(void);
 char * expand_plist_variables(const char *in, kvlist_t *vars);
 
-int scan_system_shlibs(pkghash **system_shlibs, const char *rootdir);
+int scan_system_shlibs(charv_t *system_shlibs, const char *rootdir);
 void pkg_lists_sort(struct pkg *p);
 void pkg_cleanup_shlibs_required(struct pkg *pkg, charv_t *internal_provided);
 
